import telebot
from telebot.types import LabeledPrice, ShippingOption

token = "1234567890:AAAABBBBCCCCDDDDeeeeFFFFgggGHHHH"
provider_token = (
    "1234567890:TEST:AAAABBBBCCCCDDDD"  # @BotFather -> Bot Settings -> Payments
)
bot = telebot.TeleBot(token)

# More about Payments: https://core.telegram.org/bots/payments

prices = [
    LabeledPrice(label="Working Time Machine", amount=5750),
    LabeledPrice("Gift wrapping", 500),
]

shipping_options = [
    ShippingOption(id="instant", title="WorldWide Teleporter").add_price(
        LabeledPrice("Teleporter", 1000)
    ),
    ShippingOption(id="pickup", title="Local pickup").add_price(
        LabeledPrice("Pickup", 300)
    ),
]


@bot.message_handler(commands=["start"])
def command_start(message):
    bot.send_message(
        message.chat.id,
        "Hello, I'm the demo merchant bot."
        " I can sell you a Time Machine."
        " Use /buy to order one, /terms for Terms and Conditions",
    )


@bot.message_handler(commands=["terms"])
def command_terms(message):
    bot.send_message(
        message.chat.id,
        "Thank you for shopping with our demo bot. We hope you like your new time machine!\n"
        "1. If your time machine was not delivered on time, please rethink your concept of time and try again.\n"
        "2. If you find that your time machine is not working, kindly contact our future service workshops on Trappist-1e."
        " They will be accessible anywhere between May 2075 and November 4000 C.E.\n"
        "3. If you would like a refund, kindly apply for one yesterday and we will have sent it to you immediately.",
    )


@bot.message_handler(commands=["buy"])
def command_pay(message):
    bot.send_message(
        message.chat.id,
        "Real cards won't work with me, no money will be debited from your account."
        " Use this test card number to pay for your Time Machine: `4242 4242 4242 4242`"
        "\n\nThis is your demo invoice:",
        parse_mode="Markdown",
    )
    bot.send_invoice(
        message.chat.id,  # chat_id
        "Working Time Machine",  # title
        " Want to visit your great-great-great-grandparents? Make a fortune at the races? Shake hands with Hammurabi and take a stroll in the Hanging Gardens? Order our Working Time Machine today!",  # description
        "HAPPY FRIDAYS COUPON",  # invoice_payload
        provider_token,  # provider_token
        "usd",  # currency
        prices,  # prices
        photo_url="http://erkelzaar.tsudao.com/models/perrotta/TIME_MACHINE.jpg",
        photo_height=512,  # !=0/None or picture won't be shown
        photo_width=512,
        photo_size=512,
        is_flexible=False,  # True If you need to set up Shipping Fee
        start_parameter="time-machine-example",
    )


@bot.shipping_query_handler(func=lambda query: True)
def shipping(shipping_query):
    print(shipping_query)
    bot.answer_shipping_query(
        shipping_query.id,
        ok=True,
        shipping_options=shipping_options,
        error_message="Oh, seems like our Dog couriers are having a lunch right now. Try again later!",
    )


@bot.pre_checkout_query_handler(func=lambda query: True)
def checkout(pre_checkout_query):
    bot.answer_pre_checkout_query(
        pre_checkout_query.id,
        ok=True,
        error_message="Aliens tried to steal your card's CVV, but we successfully protected your credentials,"
        " try to pay again in a few minutes, we need a small rest.",
    )


@bot.message_handler(content_types=["successful_payment"])
def got_payment(message):
    bot.send_message(
        message.chat.id,
        "Hoooooray! Thanks for payment! We will proceed your order for `{} {}` as fast as possible! "
        "Stay in touch.\n\nUse /buy again to get a Time Machine for your friend!".format(
            message.successful_payment.total_amount / 100,
            message.successful_payment.currency,
        ),
        parse_mode="Markdown",
    )


bot.infinity_polling(skip_pending=True)



from telebot import asyncio_filters
from telebot.async_telebot import AsyncTeleBot

bot = AsyncTeleBot("TOKEN")


# Handler
@bot.message_handler(chat_types=["supergroup"], is_chat_admin=True)
async def answer_for_admin(message):
    await bot.send_message(message.chat.id, "hello my admin")


# Register filter
bot.add_custom_filter(asyncio_filters.IsAdminFilter(bot))

import asyncio

asyncio.run(bot.polling())



# This example shows you how to create a custom QWERTY keyboard using reply keyboard markup
import telebot
from telebot.types import KeyboardButton, ReplyKeyboardMarkup

TOKEN = "<your_token>"
bot = telebot.TeleBot(TOKEN)

keys = [
    "1",
    "2",
    "3",
    "4",
    "5",
    "6",
    "7",
    "8",
    "9",
    "0",
    "q",
    "w",
    "e",
    "r",
    "t",
    "y",
    "u",
    "i",
    "o",
    "p",
    "a",
    "s",
    "d",
    "f",
    "g",
    "h",
    "j",
    "k",
    "l",
    "z",
    "x",
    "c",
    "v",
    "b",
    "n",
    "m",
]
symbols = [
    "1",
    "2",
    "3",
    "4",
    "5",
    "6",
    "7",
    "8",
    "9",
    "0",
    "!",
    "@",
    "#",
    "$",
    "%",
    "^",
    "&",
    "*",
    "(",
    ")",
    "'",
    '"',
    "/",
    "\\",
    ",",
    ".",
    ";",
    ":",
]


def keyboard(key_type="Normal"):
    markup = ReplyKeyboardMarkup(row_width=10)
    if key_type == "Normal":
        row = [KeyboardButton(x) for x in keys[:10]]
        markup.add(*row)
        row = [KeyboardButton(x) for x in keys[10:20]]
        markup.add(*row)
        row = [KeyboardButton(x) for x in keys[20:29]]
        markup.add(*row)
        row = [KeyboardButton(x) for x in keys[29:]]
        markup.add(*row)
        markup.add(
            KeyboardButton("Caps Lock"),
            KeyboardButton("Symbols"),
            KeyboardButton("🔙Delete"),
            KeyboardButton("✅Done"),
        )
    elif key_type == "Symbols":
        row = [KeyboardButton(x) for x in symbols[:10]]
        markup.add(*row)
        row = [KeyboardButton(x) for x in symbols[10:20]]
        markup.add(*row)
        row = [KeyboardButton(x) for x in symbols[20:]]
        markup.add(*row)
        markup.add(
            KeyboardButton("Caps Lock"),
            KeyboardButton("Normal"),
            KeyboardButton("🔙Delete"),
            KeyboardButton("✅Done"),
        )
    else:
        row = [KeyboardButton(x.upper()) for x in keys[:10]]
        markup.add(*row)
        row = [KeyboardButton(x.upper()) for x in keys[10:20]]
        markup.add(*row)
        row = [KeyboardButton(x.upper()) for x in keys[20:29]]
        markup.add(*row)
        row = [KeyboardButton(x.upper()) for x in keys[29:]]
        markup.add(*row)
        markup.add(
            KeyboardButton("Normal"),
            KeyboardButton("Symbols"),
            KeyboardButton("🔙Delete"),
            KeyboardButton("✅Done"),
        )
    return markup


@bot.message_handler(commands=["start"])
def start_message(message):
    bot.send_message(
        message.chat.id, "You can use the keyboard", reply_markup=keyboard()
    )


@bot.message_handler(func=lambda message: True)
def all_messages(message):
    if message.text == "✅Done":
        markup = telebot.types.ReplyKeyboardRemove()
        bot.send_message(
            message.from_user.id, "Done with Keyboard", reply_markup=markup
        )
    elif message.text == "Symbols":
        bot.send_message(
            message.from_user.id, "Special characters", reply_markup=keyboard("Symbols")
        )
    elif message.text == "Normal":
        bot.send_message(
            message.from_user.id, "Normal Keyboard", reply_markup=keyboard("Normal")
        )
    elif message.text == "Caps Lock":
        bot.send_message(
            message.from_user.id, "Caps Lock", reply_markup=keyboard("Caps")
        )
    elif message.text == "🔙Delete":
        bot.delete_message(message.from_user.id, message.message_id)
    else:
        bot.send_message(message.chat.id, message.text)


bot.infinity_polling()



#!/usr/bin/env python
"""
Asynchronous Telegram Echo Bot example.

This is a simple bot that echoes each message that is received onto the chat.
It uses the Starlette ASGI framework to receive updates via webhook requests.
"""

import uvicorn
from starlette.applications import Starlette
from starlette.requests import Request
from starlette.responses import PlainTextResponse, Response
from starlette.routing import Route

from telebot.async_telebot import AsyncTeleBot
from telebot.types import Message, Update

API_TOKEN = "TOKEN"

WEBHOOK_HOST = "<ip/domain>"
WEBHOOK_PORT = 8443  # 443, 80, 88 or 8443 (port need to be 'open')
WEBHOOK_LISTEN = "0.0.0.0"
WEBHOOK_SSL_CERT = "./webhook_cert.pem"  # Path to the ssl certificate
WEBHOOK_SSL_PRIV = "./webhook_pkey.pem"  # Path to the ssl private key
WEBHOOK_URL = f"https://{WEBHOOK_HOST}:{WEBHOOK_PORT}/telegram"
WEBHOOK_SECRET_TOKEN = "SECRET_TOKEN"

logger = telebot.logger
telebot.logger.setLevel(logging.INFO)

bot = AsyncTeleBot(token=API_TOKEN)


# BOT HANDLERS
@bot.message_handler(commands=["help", "start"])
async def send_welcome(message: Message):
    """
    Handle '/start' and '/help'
    """
    await bot.reply_to(
        message,
        ("Hi there, I am EchoBot.\n" "I am here to echo your kind words back to you."),
    )


@bot.message_handler(func=lambda _: True, content_types=["text"])
async def echo_message(message: Message):
    """
    Handle all other messages
    """
    await bot.reply_to(message, message.text)


# WEBSERVER HANDLERS
async def telegram(request: Request) -> Response:
    """Handle incoming Telegram updates."""
    token_header_name = "X-Telegram-Bot-Api-Secret-Token"
    if request.headers.get(token_header_name) != WEBHOOK_SECRET_TOKEN:
        return PlainTextResponse("Forbidden", status_code=403)
    await bot.process_new_updates([Update.de_json(await request.json())])
    return Response()


async def startup() -> None:
    """Register webhook for telegram updates."""
    webhook_info = await bot.get_webhook_info(30)
    if WEBHOOK_URL != webhook_info.url:
        logger.debug(
            f"updating webhook url, old: {webhook_info.url}, new: {WEBHOOK_URL}"
        )
        if not await bot.set_webhook(
            url=WEBHOOK_URL, secret_token=WEBHOOK_SECRET_TOKEN
        ):
            raise RuntimeError("unable to set webhook")


app = Starlette(
    routes=[
        Route("/telegram", telegram, methods=["POST"]),
    ],
    on_startup=[startup],
)


uvicorn.run(
    app,
    host=WEBHOOK_HOST,
    port=WEBHOOK_LISTEN,
    ssl_certfile=WEBHOOK_SSL_CERT,
    ssl_keyfile=WEBHOOK_SSL_PRIV,
)



import telebot
from telebot import AdvancedCustomFilter, types
from telebot.callback_data import CallbackData, CallbackDataFilter

calendar_factory = CallbackData("year", "month", prefix="calendar")
calendar_zoom = CallbackData("year", prefix="calendar_zoom")


class CalendarCallbackFilter(AdvancedCustomFilter):
    key = "calendar_config"

    def check(self, call: types.CallbackQuery, config: CallbackDataFilter):
        return config.check(query=call)


class CalendarZoomCallbackFilter(AdvancedCustomFilter):
    key = "calendar_zoom_config"

    def check(self, call: types.CallbackQuery, config: CallbackDataFilter):
        return config.check(query=call)


def bind_filters(bot: telebot.TeleBot):
    bot.add_custom_filter(CalendarCallbackFilter())
    bot.add_custom_filter(CalendarZoomCallbackFilter())



# -*- coding: utf-8 -*-
"""
This Example will show you how to use CallbackData
"""

from telebot import types
from telebot.async_telebot import AsyncTeleBot
from telebot.asyncio_filters import AdvancedCustomFilter
from telebot.callback_data import CallbackData, CallbackDataFilter

API_TOKEN = "TOKEN"
PRODUCTS = [
    {"id": "0", "name": "xiaomi mi 10", "price": 400},
    {"id": "1", "name": "samsung s20", "price": 800},
    {"id": "2", "name": "iphone 13", "price": 1300},
]

bot = AsyncTeleBot(API_TOKEN)
products_factory = CallbackData("product_id", prefix="products")


def products_keyboard():
    return types.InlineKeyboardMarkup(
        keyboard=[
            [
                types.InlineKeyboardButton(
                    text=product["name"],
                    callback_data=products_factory.new(product_id=product["id"]),
                )
            ]
            for product in PRODUCTS
        ]
    )


def back_keyboard():
    return types.InlineKeyboardMarkup(
        keyboard=[[types.InlineKeyboardButton(text="⬅", callback_data="back")]]
    )


class ProductsCallbackFilter(AdvancedCustomFilter):
    key = "config"

    async def check(self, call: types.CallbackQuery, config: CallbackDataFilter):
        return config.check(query=call)


@bot.message_handler(commands=["products"])
async def products_command_handler(message: types.Message):
    await bot.send_message(
        message.chat.id, "Products:", reply_markup=products_keyboard()
    )


# Only product with field - product_id = 2
@bot.callback_query_handler(func=None, config=products_factory.filter(product_id="2"))
async def product_one_callback(call: types.CallbackQuery):
    await bot.answer_callback_query(
        callback_query_id=call.id, text="Not available :(", show_alert=True
    )


# Any other products
@bot.callback_query_handler(func=None, config=products_factory.filter())
async def products_callback(call: types.CallbackQuery):
    callback_data: dict = products_factory.parse(callback_data=call.data)
    product_id = int(callback_data["product_id"])
    product = PRODUCTS[product_id]

    text = f"Product name: {product['name']}\n" f"Product price: {product['price']}"
    await bot.edit_message_text(
        chat_id=call.message.chat.id,
        message_id=call.message.message_id,
        text=text,
        reply_markup=back_keyboard(),
    )


@bot.callback_query_handler(func=lambda c: c.data == "back")
async def back_callback(call: types.CallbackQuery):
    await bot.edit_message_text(
        chat_id=call.message.chat.id,
        message_id=call.message.message_id,
        text="Products:",
        reply_markup=products_keyboard(),
    )


bot.add_custom_filter(ProductsCallbackFilter())
import asyncio

asyncio.run(bot.polling())



import os
import pickle

from telebot.storage.base_storage import StateContext, StateStorageBase


class StatePickleStorage(StateStorageBase):
    def __init__(self, file_path="./.state-save/states.pkl") -> None:
        super().__init__()
        self.file_path = file_path
        self.create_dir()
        self.data = self.read()

    def convert_old_to_new(self):
        """
        Use this function to convert old storage to new storage.
        This function is for people who was using pickle storage
        that was in version <=4.3.1.
        """
        # old looks like:
        # {1: {'state': 'start', 'data': {'name': 'John'}}
        # we should update old version pickle to new.
        # new looks like:
        # {1: {2: {'state': 'start', 'data': {'name': 'John'}}}}
        new_data = {}
        for key, value in self.data.items():
            # this returns us id and dict with data and state
            new_data[key] = {key: value}  # convert this to new
        # pass it to global data
        self.data = new_data
        self.update_data()  # update data in file

    def create_dir(self):
        """
        Create directory .save-handlers.
        """
        dirs, filename = os.path.split(self.file_path)
        os.makedirs(dirs, exist_ok=True)
        if not os.path.isfile(self.file_path):
            with open(self.file_path, "wb") as file:
                pickle.dump({}, file)

    def read(self):
        file = open(self.file_path, "rb")
        data = pickle.load(file)
        file.close()
        return data

    def update_data(self):
        file = open(self.file_path, "wb+")
        pickle.dump(self.data, file, protocol=pickle.HIGHEST_PROTOCOL)
        file.close()

    def set_state(self, chat_id, user_id, state):
        if hasattr(state, "name"):
            state = state.name
        if chat_id in self.data:
            if user_id in self.data[chat_id]:
                self.data[chat_id][user_id]["state"] = state
                self.update_data()
                return True
            else:
                self.data[chat_id][user_id] = {"state": state, "data": {}}
                self.update_data()
                return True
        self.data[chat_id] = {user_id: {"state": state, "data": {}}}
        self.update_data()
        return True

    def delete_state(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                del self.data[chat_id][user_id]
                if chat_id == user_id:
                    del self.data[chat_id]
                self.update_data()
                return True

        return False

    def get_state(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                return self.data[chat_id][user_id]["state"]

        return None

    def get_data(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                return self.data[chat_id][user_id]["data"]

        return None

    def reset_data(self, chat_id, user_id):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                self.data[chat_id][user_id]["data"] = {}
                self.update_data()
                return True
        return False

    def set_data(self, chat_id, user_id, key, value):
        if self.data.get(chat_id):
            if self.data[chat_id].get(user_id):
                self.data[chat_id][user_id]["data"][key] = value
                self.update_data()
                return True
        raise RuntimeError(
            "chat_id {} and user_id {} does not exist".format(chat_id, user_id)
        )

    def get_interactive_data(self, chat_id, user_id):
        return StateContext(self, chat_id, user_id)

    def save(self, chat_id, user_id, data):
        self.data[chat_id][user_id]["data"] = data
        self.update_data()
